<?xml version="1.0"?>
<xsl:stylesheet 
  version='1.0' 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  extension-element-prefixes="exslt">

  <!-- Convert sections of an EtreCheck report into HTML 
       representations. -->
  <xsl:output method="html" indent="yes" encoding="UTF-8"/> 
 
  <!-- French isn't done yet. -->
  <xsl:param name="language">en</xsl:param>
  
  <!-- Language-specific data. -->
  <xsl:variable 
    name="strings" 
    select="document(concat('html_', $language, '.xml'))/strings"/>
  
  <!-- Language-specific issues. -->
  <xsl:variable 
    name="issues" 
    select="document(concat('Issues_', $language, '.xml'))/issues"/>

  <!-- Grab some convenient lookup tables. -->
  <xsl:variable name="performance" select="$strings/performance_rating"/>

  <xsl:variable name="batteryhealth" select="$strings/batteryhealth"/>

  <xsl:variable name="statusattributes" select="$strings/status"/>
  
  <!-- Newlines are special in XSL. -->
  <xsl:variable name="newline">
    <xsl:text>&#xa;</xsl:text>
  </xsl:variable>

  <!-- Is is only for the full report. -->
  <xsl:template match='/etrecheck'>
  
    <xsl:text disable-output-escaping='yes'>&lt;!DOCTYPE html&gt;
</xsl:text>
    <html>
      <xsl:attribute name="lang">
        <xsl:value-of select="$language"/>
      </xsl:attribute>
    
      <head>

        <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>

        <title>EtreCheck Report</title>

        <!-- Frameworks stuff. -->
        <script src="/script/jquery.js"></script>

        <script src="/script/etrecheck.js"></script>

      </head>

      <body>
      
        <xsl:apply-templates select="header"/>
        <xsl:call-template name="major"/>
        <xsl:call-template name="minor"/>
        <xsl:apply-templates select="hardware"/>
        <xsl:apply-templates select="video"/>
        <xsl:apply-templates select="disk"/>
        <xsl:apply-templates select="virtualvolume"/>
        <xsl:apply-templates select="network"/>
        <xsl:apply-templates select="systemsoftware"/>
        <xsl:apply-templates select="configurationfiles"/>
        <xsl:apply-templates select="security"/>
        <xsl:apply-templates select="adware"/>
        <xsl:apply-templates select="unsigned"/>
        <xsl:apply-templates select="kernelextensions"/>
        <xsl:apply-templates select="startupitems"/>
        <xsl:apply-templates select="systemlaunchagents"/>
        <xsl:apply-templates select="systemlaunchdaemons"/>
        <xsl:apply-templates select="launchagents"/>
        <xsl:apply-templates select="launchdaemons"/>
        <xsl:apply-templates select="userlaunchagents"/>
        <xsl:apply-templates select="loginitems"/>
        <xsl:apply-templates select="internetplugins"/>
        <xsl:apply-templates select="userinternetplugins"/>
        <xsl:apply-templates select="audioplugins"/>
        <xsl:apply-templates select="useraudioplugins"/>
        <xsl:apply-templates select="safariextensions"/>
        <xsl:apply-templates select="preferencepanes"/>
        <xsl:apply-templates select="timemachine"/>
        <xsl:apply-templates select="cpu"/>
        <xsl:apply-templates select="memory"/>
        <xsl:apply-templates select="networkusage"/>
        <xsl:apply-templates select="energy"/>
        <xsl:apply-templates select="vm"/>
        <xsl:apply-templates select="install"/>
        <xsl:apply-templates select="cleanup"/>
        <xsl:apply-templates select="diagnostics"/>
        <xsl:apply-templates select="etrecheckdeletedfiles"/>
        
        <!-- Add some plain text at the end to keep the ASC edit window
             from going into a funky table-edit mode. -->
        <br/>
        <p>
          <xsl:call-template name="style"/>
          <strong>
            <xsl:value-of select="$strings/endofreport"/>
          </strong>
        </p>
            
      </body>

    </html>

  </xsl:template>

  <!-- EtreCheck stats. -->
  <xsl:template match="header">
  
    <xsl:variable name="performance" select="performance"/>
    
    <p>
      <xsl:call-template name="style"/>
      
      <xsl:apply-templates select="$strings/header/version" mode="label"/>
      <strong>
        <xsl:value-of select="version"/>
        <xsl:text> (</xsl:text>
        <xsl:value-of select="build"/>
        <xsl:text>)</xsl:text>
      </strong>
    </p>
    
    <p>
      <xsl:call-template name="style"/>
            
      <xsl:apply-templates 
        select="$strings/header/report_generated" mode="label"/>

      <strong>
        <xsl:value-of select="date"/>
      </strong>
    </p>
    
    <p>
      <xsl:call-template name="style"/>

      <xsl:apply-templates 
        select="$strings/header/download_etrecheck" mode="label"/>
    </p>

    <p>
      <xsl:call-template name="style"/>
      
      <xsl:apply-templates select="$strings/header/runtime" mode="label"/>
      <strong>
        <xsl:value-of select="runtime"/>
      </strong>
    </p>

    <p>
      <xsl:call-template name="style"/>
      
      <xsl:apply-templates 
        select="$strings/header/performance" mode="label"/>
      <strong>
        <xsl:value-of 
          select="$strings/header/performance_rating/value[@key = $performance]"/>
      </strong>
    </p>

    <br/>
    
    <p>
      <xsl:call-template name="style"/>
      
      <xsl:apply-templates select="$strings/header/problem" mode="label"/>
      <xsl:value-of select="problem"/>
    </p>

    <!-- Description is optional. -->
    <xsl:if test="problemdescription">
      <p>
        <xsl:call-template name="style"/>
        
        <xsl:apply-templates 
          select="$strings/header/problemdescription" mode="label"/>
      </p>
      
      <!-- ASC scrambles multi-line elements. Do the scrambling myself so
           I have some control over the outcome. -->
      <xsl:call-template name="split">
        <xsl:with-param name="text">
          <xsl:value-of select="problemdescription"/>
        </xsl:with-param>
      </xsl:call-template>
      
    </xsl:if>
    
  </xsl:template>

  <xsl:include href="Issues.xsl" />

  <!-- Major issues. -->
  <xsl:template name="major">
  
    <xsl:variable name="content">
      <xsl:call-template name="majorissues"/>
    </xsl:variable>

    <xsl:choose>
    
      <xsl:when test="string-length($content) = 0">
        <xsl:call-template name="custom_header">
          <xsl:with-param name="text">
            <xsl:value-of select="$issues/major/none/node()"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:call-template name="header">
          <xsl:with-param name="key">major</xsl:with-param>
        </xsl:call-template>
        
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">20</xsl:with-param>
          </xsl:call-template>

          <xsl:copy-of select="$issues/major/problemsfound/node()"/>
        </p>
        <br/>

        <xsl:copy-of select="$content"/>
          
      </xsl:otherwise>
    
    </xsl:choose>
  
  </xsl:template>
    
  <!-- Minor issues. -->
  <xsl:template name="minor">
  
    <xsl:variable name="content">
      <xsl:call-template name="minorissues"/>
    </xsl:variable>

    <xsl:choose>
    
      <xsl:when test="string-length($content) = 0">
        <xsl:call-template name="custom_header">
          <xsl:with-param name="text">
            <xsl:value-of select="$issues/minor/none/node()"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:call-template name="header">
          <xsl:with-param name="key">minor</xsl:with-param>
        </xsl:call-template>
        
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">20</xsl:with-param>
          </xsl:call-template>

          <xsl:copy-of select="$issues/minor/problemsfound/node()"/>
        </p>
        <br/>

        <xsl:copy-of select="$content"/>
          
      </xsl:otherwise>
    
    </xsl:choose>

  </xsl:template>
  
  <!-- Emit an issue. -->
  <xsl:template name="issue">
    <xsl:param name="type"/>
    <xsl:param name="name"/>
    
    <xsl:variable 
      name="issuenodes" select="$issues/*[local-name() = $type]"/>
    
    <xsl:variable 
      name="issue" select="$issuenodes/*[local-name() = $name]"/>
    
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent">20</xsl:with-param>
      </xsl:call-template>

      <strong><xsl:value-of select="$issue/title"/></strong>
      <xsl:text> - </xsl:text>
      <xsl:value-of select="$issue/description"/>
    </p>

  </xsl:template>

  <!-- Hardware information. -->
  <xsl:template match="hardware">
  
    <xsl:variable name="healthvalue" select="batteryhealth"/>

    <xsl:call-template name="header"/>

    <p>
      <xsl:call-template name="style"/>
      
      <xsl:value-of select="marketingname"/>

      <xsl:choose>
        <xsl:when test="vintage = 'vintage'">
          <xsl:text> - </xsl:text>
          <xsl:copy-of select="$strings/hardware/vintage/node()"/>
        </xsl:when>
        
        <xsl:when test="vintage = 'obsolete'">
          <xsl:text> - </xsl:text>
          <xsl:copy-of select="$strings/hardware/obsolete/node()"/>
        </xsl:when>

        <xsl:otherwise/>
        
      </xsl:choose>

    </p>

    <p>
      <xsl:call-template name="style"/>
      
      <xsl:value-of select="name"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="$strings/hardware/model"/>
      <xsl:value-of select="model"/>
    </p>

    <p>
      <xsl:call-template name="style"/>
      
      <xsl:value-of select="cpucount"/>
      <xsl:text> </xsl:text>
      <xsl:apply-templates select="speed" mode="units"/>
      <xsl:text> </xsl:text>
      
      <xsl:call-template name="format">
        <xsl:with-param name="format" 
          select="$strings/hardware/cpu_format"/>
        <xsl:with-param name="value" select="cpu_type|cpucode|corecount"/>
      </xsl:call-template>
    </p>

    <p>
      <xsl:call-template name="style"/>
      
      <span>
        <xsl:apply-templates select="ram"/>
        <xsl:text> RAM</xsl:text>
      </span>      
       
      <xsl:text> </xsl:text>
      
      <xsl:choose> 
      
        <xsl:when test="upgradeable = 'true'">     
          <xsl:value-of select="$strings/hardware/upgradeable"/>
        </xsl:when>
        
        <xsl:otherwise>
          <xsl:value-of select="$strings/hardware/not_upgradeable"/>
        </xsl:otherwise>
        
      </xsl:choose>
      
    </p>

    <xsl:for-each select="memorybanks/memorybank">
      <p>
        <xsl:call-template name="style"/>
        <xsl:value-of select="identifier"/>
      </p>
      <p>
        <xsl:call-template name="style">
          <xsl:with-param name="indent">20</xsl:with-param>
        </xsl:call-template>
        <xsl:apply-templates select="size" mode="units"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="type"/>
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="speed" mode="units"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="status"/>
      </p>
    </xsl:for-each>
    
    <xsl:if test="batteryhealth and batterycyclecount and batterypercent">
      <p>
        <xsl:call-template name="style"/>
        
        <xsl:value-of select="$strings/hardware/battery"/>
        <xsl:value-of select="$strings/hardware/health"/>
        <xsl:text> = </xsl:text>
        <xsl:value-of 
          select="$strings/hardware/batteryhealth/value[@key = $healthvalue]"/>
        <xsl:text> - </xsl:text>
        <xsl:value-of select="$strings/hardware/cycle_count"/>
        <xsl:text> = </xsl:text>
        <xsl:value-of select="batterycyclecount"/>
      </p>
    </xsl:if>
    
  </xsl:template>
 
  <!-- Video information. -->
  <xsl:template match="video">
  
    <xsl:call-template name="header"/>

    <xsl:for-each select="gpu">
      <p>
        <xsl:call-template name="style"/>
        
        <xsl:value-of select="name"/>
        <xsl:if test="vram">
          <xsl:text> - VRAM: </xsl:text>
          <xsl:apply-templates select="vram" mode="units"/>
        </xsl:if>
      </p>
    
      <!-- Report displays, if any. -->
      <xsl:if test="count(displays/display) &gt; 0">
        <xsl:for-each select="displays/display">
          <p>
            <xsl:call-template name="style"/>
            
            <xsl:value-of select="name"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="resolution"/>
          </p>
        </xsl:for-each>
      </xsl:if>
    </xsl:for-each>
      
  </xsl:template>
  
  <!-- Drive information. -->
  <xsl:template match="disk">
  
    <xsl:call-template name="header"/>

    <xsl:apply-templates select="drive"/>
    
  </xsl:template>

  <!-- Physical drive. -->
  <xsl:template match="drive">
  
    <xsl:variable name="type" select="type"/>
    <xsl:variable name="mechanism" select="mechanism"/>
    
    <xsl:if test="position() &gt; 1">
      <br/>
    </xsl:if>
    
    <p>
      <xsl:call-template name="style"/>      

      <xsl:value-of select="device"/>
      <xsl:text> - </xsl:text>
      <xsl:value-of select="model"/>
      <xsl:text> </xsl:text>
      <xsl:apply-templates select="size" mode="units"/>
      
      <xsl:if test="mechanism != 'unknown'">
        <xsl:text> (</xsl:text>
        <xsl:value-of
          select="$strings/drive/mechanisms/value[@key = $mechanism]"/>
        
        <xsl:if test="TRIM">
          <xsl:value-of select="$strings/drive/trim"/>
          <xsl:apply-templates select="TRIM"/>
        </xsl:if>
        
        <xsl:text>)</xsl:text>
      </xsl:if>
    </p>
    
    <p>
      <xsl:call-template name="style"/>
      
      <xsl:choose>
        <xsl:when test="internal = 'true'">
          <xsl:value-of select="$strings/drive/internal"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$strings/drive/external"/>
        </xsl:otherwise>
      </xsl:choose>
      
      <xsl:text> </xsl:text>
      <xsl:value-of select="bus"/>
        
      <xsl:if test="busspeed">
        <xsl:text> </xsl:text>
        <xsl:value-of select="busspeed"/>
      </xsl:if>

      <xsl:if test="type">
        <xsl:text> </xsl:text>
        <xsl:value-of select="$strings/drive/types/value[@key = $type]"/>
      </xsl:if>
      
    </p>
    
    <xsl:choose>
      <xsl:when test="smartstatus = 'Verified'"/>
      <xsl:when test="smartstatus = 'Not Supported'"/>
      <xsl:otherwise>
        <p>
          <xsl:call-template name="style"/>
        
          <xsl:value-of select="$strings/drive/smart_status"/>
          <xsl:value-of select="smartstatus"/>
        </p>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:if test="volumes/device">
      <xsl:call-template name="volumes">
        <xsl:with-param name="volumes" select="volumes"/>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>

  <!-- Other volumes information. -->
  <xsl:template match="virtualvolume">
  
    <xsl:variable name="othervolumes">
      <xsl:for-each select="volume">
        <xsl:variable name="device" select="device"/>
        <xsl:choose>
          <xsl:when test="mountpoint">
            <xsl:copy-of select="."/>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
      </xsl:for-each>
    </xsl:variable>
    
    <xsl:if test="count($othervolumes) &gt; 0">
    
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="exslt:node-set($othervolumes)"/>
      
    </xsl:if>
    
  </xsl:template>

  <!-- A mini-list of devices. -->
  <xsl:template name="volumes">
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="volumes"/>
  
    <xsl:for-each select="$volumes/device">
      <xsl:call-template name="device">
        <xsl:with-param name="indent" select="$indent"/>
        <xsl:with-param name="device" select="."/>
      </xsl:call-template>
    </xsl:for-each>
    
  </xsl:template>
  
  <!-- A mini-list of devices. -->
  <xsl:template name="device">
    <xsl:param name="indent">10</xsl:param>
    <xsl:param name="device"/>
    
    <xsl:variable 
      name="volume" 
      select="/etrecheck/virtualvolume/volume[device = $device]"/>
    
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent" select="$indent"/>
      </xsl:call-template>
    
      <xsl:value-of select="$device"/>

      <xsl:if test="$volume/cleanname">
        <xsl:text> - </xsl:text>
        <xsl:value-of select="$volume/cleanname"/>
      </xsl:if>

      <xsl:if test="$volume/filesystem">
        
        <xsl:text> (</xsl:text>
        <xsl:value-of select="$volume/filesystem"/>
        <xsl:text>)</xsl:text>
      
      </xsl:if>

      <xsl:choose>
        
        <xsl:when test="$volume/type">
          <xsl:text> [</xsl:text>
          <xsl:value-of 
            select="$strings/volume/types/value[@key = $volume/type]"/>
          <xsl:text>]</xsl:text>
        </xsl:when>
        
        <xsl:otherwise/>
      </xsl:choose>
      
      <xsl:text> </xsl:text>
      <xsl:apply-templates select="$volume/size" mode="units"/>

      <xsl:if test="$volume/used &gt; 0">
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="$volume/used" mode="units"/>
      </xsl:if>
      
    </p>
      
    <xsl:if test="$volume/volumes/device">
      <xsl:call-template name="volumes">
        <xsl:with-param name="indent" select="$indent + 20"/>
        <xsl:with-param name="volumes" select="$volume/volumes"/>
      </xsl:call-template>
    </xsl:if>
    
  </xsl:template>

  <!-- Storage volume. -->
  <xsl:template match="volume">
  
    <xsl:variable name="type" select="type"/>

    <xsl:if test="position() &gt; 1">
      <br/>
    </xsl:if>
    
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent">20</xsl:with-param>
      </xsl:call-template>
      
      <xsl:value-of select="device"/>
      
      <xsl:if test="cleanname">
        <xsl:text> - </xsl:text>
        <xsl:value-of select="cleanname"/>
      </xsl:if>
      
      <xsl:if test="type">
        <xsl:text> [</xsl:text>
        <xsl:value-of select="$strings/volume/types/value[@key = $type]"/>
        <xsl:text>] </xsl:text>
      </xsl:if>
    
      <xsl:text> </xsl:text>
      <xsl:apply-templates select="size" mode="units"/>
      
      <xsl:if test="free &gt; 0">
        <xsl:text> (</xsl:text>
        <xsl:apply-templates select="free" mode="units"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="$strings/volume/free"/>
        <xsl:text>)</xsl:text>
      </xsl:if>

    </p>
    
    <xsl:if test="filesystem">
      <p>
        <xsl:call-template name="style">
          <xsl:with-param name="indent">40</xsl:with-param>
        </xsl:call-template>
      
        <xsl:value-of select="filesystem"/>
      </p>
    </xsl:if>
    
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent">40</xsl:with-param>
      </xsl:call-template>
    
      <xsl:choose>
        <xsl:when test="mountpoint">
          <xsl:value-of select="$strings/volume/mountpoint"/>
          <xsl:value-of select="cleanmountpoint"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$strings/volume/not_mounted"/>
        </xsl:otherwise>
      </xsl:choose>
    </p>
    
    <xsl:if test="encryption">
      <p>
        <xsl:call-template name="style">
          <xsl:with-param name="indent">40</xsl:with-param>
        </xsl:call-template>
    
        <xsl:variable 
          name="encryptionstatus" select="encryption/status"/>
        
        <xsl:value-of 
          select="$strings/volume/encryption/value[@key = $encryptionstatus]"/>
          
        <xsl:if test="encryption/status != 'encrypted'">
          <xsl:text> </xsl:text>
          <xsl:value-of select="encryption/progress"/>
          <xsl:value-of select="$strings/volume/percent_done"/>
        </xsl:if>
      </p>
    </xsl:if>
    
    <xsl:if test="errors">
      <p>
        <xsl:call-template name="style">
          <xsl:with-param name="indent">20</xsl:with-param>
        </xsl:call-template>
        
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/volume"/>
          <xsl:with-param name="key">
            <xsl:text>errors</xsl:text>
          </xsl:with-param>
          <xsl:with-param name="value" select="errors"/>
        </xsl:call-template>
      </p>
    </xsl:if>
    
    <xsl:if test="volumes/device">
      <p>
        <xsl:call-template name="style">
          <xsl:with-param name="indent">20</xsl:with-param>
        </xsl:call-template>
      
        <xsl:value-of select="$strings/volume/partitions"/>
      </p>
      <xsl:call-template name="volumes">
        <xsl:with-param name="indent">20</xsl:with-param>
        <xsl:with-param name="volumes" select="volumes"/>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>

  <!-- Network information. -->
  <xsl:template match="network">
  
    <xsl:call-template name="header"/>

    <xsl:for-each select="interfaces/networkinterface">
      <xsl:variable name="interface" select="interface"/>
      <xsl:variable 
        name="modes" 
        select="/etrecheck/hardware/wireless[name = $interface]"/>

      <p>
        <xsl:call-template name="style">
          <xsl:with-param name="indent">-10</xsl:with-param>
        </xsl:call-template>
      
        <xsl:value-of select="$strings/network/interface"/>
        <xsl:value-of select="interface"/>
        <xsl:value-of select="$strings/network/colon"/>
        <xsl:value-of select="name"/>
      </p>
      
      <xsl:if test="$modes">
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">10</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$modes/modes"/>
        </p>
      </xsl:if>

      <xsl:if test="count(ipv4addresses/address) &gt; 0">
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">10</xsl:with-param>
          </xsl:call-template>
        
          <xsl:call-template name="plural">
            <xsl:with-param name="keys" select="$strings/network"/>
            <xsl:with-param name="key">
              <xsl:text>ipv4address</xsl:text>
            </xsl:with-param>
            <xsl:with-param 
              name="value" 
              select="exslt:node-set(count(ipv4addresses/address))"/>
          </xsl:call-template>
        </p>
      </xsl:if>
      
      <xsl:if test="count(ipv6addresses/address) &gt; 0">
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">10</xsl:with-param>
          </xsl:call-template>
        
          <xsl:call-template name="plural">
            <xsl:with-param name="keys" select="$strings/network"/>
            <xsl:with-param name="key">
              <xsl:text>ipv6address</xsl:text>
            </xsl:with-param>
            <xsl:with-param 
              name="value" 
              select="exslt:node-set(count(ipv6addresses/address))"/>
          </xsl:call-template>
        </p>
      </xsl:if>
      
      <xsl:if test="proxyautodiscovery = 'true'">
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">10</xsl:with-param>
          </xsl:call-template>
        
          <xsl:value-of select="$strings/network/proxyautodiscovery"/>
        </p>
      </xsl:if>
      
      <xsl:if test="proxyautoconfig = 'true'">
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">10</xsl:with-param>
          </xsl:call-template>
        
          <xsl:value-of select="$strings/network/proxyautoconfig"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="$strings/network/proxyautoconfigurl"/>
        </p>
      </xsl:if>

      <xsl:if test="proxies/proxy">
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">10</xsl:with-param>
          </xsl:call-template>
        
          <xsl:value-of select="$strings/network/proxies"/>
          <xsl:for-each select="proxies/proxy">
            <xsl:if test="position() &gt; 1">
              <xsl:text>, </xsl:text>
            </xsl:if>
            
            <xsl:value-of select="type"/>
          </xsl:for-each>
        </p>
      </xsl:if>

    </xsl:for-each>
    
    <xsl:if test="icloudfree">
      <p>
        <xsl:call-template name="style">
          <xsl:with-param name="indent">10</xsl:with-param>
        </xsl:call-template>
        
        <xsl:value-of select="$strings/hardware/icloudfree"/>
        <xsl:apply-templates select="icloudfree" mode="units"/>
        <xsl:value-of select="$strings/hardware/available"/>
      </p>
    </xsl:if>

    <xsl:if test="count(icloudpendingfiles//file) &gt; 0">
      <p>
        <xsl:call-template name="style">
          <xsl:with-param name="indent">10</xsl:with-param>
        </xsl:call-template>
        
        <xsl:value-of select="$strings/hardware/icloudstatus"/>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/hardware"/>
          <xsl:with-param name="key">
            <xsl:text>icloudpendingfiles</xsl:text>
          </xsl:with-param>
          <xsl:with-param 
            name="value" 
            select="exslt:node-set(count(icloudpendingfiles//file))"/>
        </xsl:call-template>
      </p>
    </xsl:if>

  </xsl:template>
  
  <!-- System software. -->
  <xsl:template match="systemsoftware">
  
    <xsl:call-template name="header"/>

    <p>
      <xsl:call-template name="style"/>
      
      <xsl:value-of select="name"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="version"/>
      <xsl:text> (</xsl:text>
      <xsl:value-of select="build"/>
      <xsl:text>) </xsl:text>
    </p>
    
    <p>
      <xsl:call-template name="style"/>
      
      <xsl:value-of select="$strings/uptime"/>
      <xsl:apply-templates select="uptime"/>
    </p>

    <p>
      <xsl:call-template name="style"/>
      
      <xsl:call-template name="format">
        <xsl:with-param name="format" 
          select="$strings/systemsoftware/systemload"/>
        <xsl:with-param name="value" select="load1|load5|load15"/>
      </xsl:call-template>

    </p>

  </xsl:template>

  <!-- Uptime. -->
  <xsl:template match="uptime">

    <xsl:variable name="days" select=". div 24"/>
    <xsl:variable name="hours" select=". mod 24"/>
    
    <xsl:choose>
    
      <xsl:when test="$days &gt;= 1">
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/uptime_human"/>
          <xsl:with-param name="key" select="'day'"/>
          <xsl:with-param name="value" select="$days"/>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/uptime_human"/>
          <xsl:with-param name="key" select="'hour'"/>
          <xsl:with-param name="value" select="$hours"/>
        </xsl:call-template>
      </xsl:otherwise>
      
    </xsl:choose>
    
  </xsl:template>

  <!-- Configuration files. -->
  <xsl:template match="configurationfiles">
  
    <xsl:if test="count(*)">
      <xsl:call-template name="header"/>
      
      <xsl:if test="unexpectedsudoerssize">
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="$strings/unexpectedsudoerssize"/>
          <xsl:text> </xsl:text>
          <xsl:apply-templates select="size" mode="units"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="$strings/but_expected"/>
          <xsl:text> </xsl:text>
          <xsl:apply-templates select="expectedsize" mode="units"/>
        </p>
      </xsl:if>
      <xsl:if test="etcsysctlconfexists">
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="$strings/etcsysctlconfexists"/>
        </p>
      </xsl:if>
      <xsl:if test="etclaunchdconfexists">
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="$strings/etclaunchdconfexists"/>
        </p>
      </xsl:if>
      <xsl:if test="hostscount">
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="$strings/hostscount"/>
          <xsl:value-of select="hostscount"/>
          
          <xsl:if test="hostscorrupt">
            <xsl:text> - </xsl:text>
            <xsl:value-of select="$strings/hostscorrupt"/>
          </xsl:if>
        </p>
      </xsl:if>
    </xsl:if>
        
  </xsl:template>

  <!-- Security information. -->
  <xsl:template match="security">
  
    <xsl:call-template name="header"/>
    
    <table>
      <xsl:call-template name="tablestyle"/>
      
      <tr>
        <th>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <strong><xsl:value-of select="$strings/security/system"/></strong>
        </th>
        <th>
          <xsl:call-template name="cellstyle"/>
          
          <strong><xsl:value-of select="$strings/security/status"/></strong>
        </th>
      </tr>
      <tr>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/security/gatekeeper"/>
        </td>
        <td>
          <xsl:call-template name="cellstyle"/>
          
          <xsl:variable name="gatekeeperstatus" select="gatekeeper/status"/>
          
          <xsl:value-of 
            select="$strings/security/gatekeeper_setting/value[@key = $gatekeeperstatus]"/>
        </td>
      </tr>
      <xsl:if test="/etrecheck/systemsoftware/majorversion &gt; 10">
        <tr>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>
            
            <xsl:value-of select="$strings/security/sip"/>
          </td>
          <td>
            <xsl:call-template name="cellstyle"/>
            
            <xsl:choose>
              <xsl:when test="SIP = 'enabled'">
                <xsl:value-of select="$strings/security/sip_enabled"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="SIP"/>
              </xsl:otherwise>
            </xsl:choose>
          </td>
        </tr>
      </xsl:if>
    </table>


    <table>
      <xsl:call-template name="tablestyle"/>
      
      <tr>
        <th/>
        <th colspan="2">
          <xsl:call-template name="cellstyle"/>
          
          <strong>
            <xsl:value-of select="$strings/security/versions"/>
          </strong>
        </th>
        <th/>
      </tr>
      <tr>
        <th>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <strong><xsl:value-of select="$strings/security/system"/></strong>
        </th>
        <th>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <strong>
            <xsl:value-of select="$strings/security/currentversion"/>
          </strong>
        </th>
        <th>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <strong>
            <xsl:value-of select="$strings/security/installedversion"/>
          </strong>
        </th>
      </tr>
      <tr>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/security/gatekeeper"/>
        </td>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <xsl:apply-templates select="gatekeeper/currentversion"/>
        </td>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <xsl:apply-templates select="gatekeeper/installedversion"/>
        </td>
      </tr>
      <tr>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/security/xprotect"/>
        </td>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <xsl:apply-templates select="xprotect/currentversion"/>
        </td>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>
          
          <xsl:apply-templates select="xprotect/installedversion"/>
        </td>
        <td/>
      </tr>
      
      <xsl:if test="/etrecheck/systemsoftware/majorversion &gt; 10">
        <tr>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>
            
            <xsl:value-of select="$strings/security/mrt"/>
          </td>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>
            
            <xsl:apply-templates select="mrt/currentversion"/>
          </td>
          <td>
            <xsl:call-template name="cellstyle"/>
            
            <xsl:apply-templates select="mrt/installedversion"/>
          </td>
          <td/>
        </tr>
      </xsl:if>
      
    </table>
      
  </xsl:template>

  <!-- Adware information. -->
  <xsl:template match="adware">
  
    <xsl:if test="count(launchdfiles/launchdfile) &gt; 0 or count(safariextensions/extension) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:for-each select="launchdfiles/launchdfile">

        <xsl:variable name="adware" select="adware"/>

        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">20</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/adware/launchd"/>
          <xsl:value-of select="path_safe"/>
        </p>
        
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">30</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/adware/reason"/>
          <xsl:value-of 
            select="$strings/adware/reason_text/value[@key = $adware]"/>
        </p>

        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">30</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/adware/executable"/>
          <xsl:value-of select="executable_safe"/>
          
          <xsl:for-each select="arguments_safe/argument">
            <xsl:value-of select="concat(' ', .)"/>
          </xsl:for-each>
        </p>

      </xsl:for-each>

      <xsl:for-each select="safariextensions/extension">

        <xsl:variable name="adware" select="adware"/>

        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">20</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/adware/safariextension"/>
          <xsl:value-of select="displayname"/>
        </p>
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">30</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/adware/reason"/>
          <xsl:value-of 
            select="$strings/adware/reason_text/value[@key = $adware]"/>
        </p>
      </xsl:for-each>

    </xsl:if>
  
  </xsl:template>

  <!-- Unsigned files information. -->
  <xsl:template match="unsigned">
  
    <xsl:if test="count(launchdfiles/launchdfile) &gt; 0 or count(safariextensions/extension) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:for-each select="launchdfiles/launchdfile">

        <xsl:variable name="details" select="details"/>
        
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">20</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/unsigned/launchd"/>
          
          <xsl:choose>
            <xsl:when test="$details = 'whitelist'">
              <xsl:value-of select="path_safe"/>
            </xsl:when>
            <xsl:when test="$details = 'whitelistprefix'">
              <xsl:value-of select="path_safe"/>
            </xsl:when>
            <xsl:otherwise>
              <strong><xsl:value-of select="path_safe"/></strong>
            </xsl:otherwise>
          </xsl:choose>
        </p>
        
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">30</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/unsigned/executable"/>
          <xsl:value-of select="executable_safe"/>
          <xsl:for-each select="arguments_safe/argument">
            <xsl:text> </xsl:text>
            <xsl:value-of select="."/>
          </xsl:for-each>
        </p>
        
        <xsl:if test="details">
          <p>
            <xsl:call-template name="style">
              <xsl:with-param name="indent">30</xsl:with-param>
            </xsl:call-template>
            
            <xsl:value-of select="$strings/unsigned/details"/>
            <xsl:value-of 
              select="$strings/unsigned/details_text/value[@key = $details]"/>
          </p>
        </xsl:if>
        
      </xsl:for-each>

      <xsl:for-each select="safariextensions/extension">

        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">20</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/adware/safariextension"/>
          <xsl:value-of select="displayname"/>
        </p>
      </xsl:for-each>

    </xsl:if>
        
  </xsl:template>

  <!-- Clean up information. -->
  <xsl:template match="cleanup">
  
    <xsl:variable 
      name="missingexecutables" 
      select="count(launchdfiles//executable) &gt; 0"/>
    
    <xsl:variable 
      name="missingextensions" 
      select="count(safariextensions//path) &gt; 0"/>

    <xsl:variable 
      name="notificationspam" 
      select="count(notificationspam/spammer) &gt; 0"/>

    <xsl:if test="$missingexecutables or $missingextensions or $notificationspam">
      <xsl:call-template name="header"/>

      <xsl:for-each select="launchdfiles/launchdfile">
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="path_safe"/>
        </p>
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">20</xsl:with-param>
          </xsl:call-template>
          <xsl:value-of select="executable_safe"/>
        </p>
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">20</xsl:with-param>
          </xsl:call-template>
          <xsl:value-of select="$strings/executable_not_found"/>
        </p>
      </xsl:for-each>

      <xsl:if test="$missingexecutables and $missingextensions">
        <br/>
      </xsl:if>
      
      <xsl:for-each select="safariextensions/extension">
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="displayname"/>
          <xsl:text> - </xsl:text>
          <xsl:value-of select="$strings/extension_not_found"/>
        </p>
        <p>
          <xsl:call-template name="style">
            <xsl:with-param name="indent">20</xsl:with-param>
          </xsl:call-template>
          
          <xsl:value-of select="path_safe"/>
        </p>
      </xsl:for-each>

      <xsl:if test="($missingexecutables or $missingextensions) and $notificationspam">
        <br/>
      </xsl:if>
      
      <xsl:for-each select="notificationspam/spammer">
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="name"/>
          <xsl:text> - </xsl:text>
          <xsl:call-template name="plural">
            <xsl:with-param name="key">spamnotificationcount</xsl:with-param>
            <xsl:with-param name="value">
              <xsl:value-of select="count"/>
            </xsl:with-param>
          </xsl:call-template>
        </p>
      </xsl:for-each>

    </xsl:if>
        
  </xsl:template>

  <!-- Kernel extensions. -->
  <xsl:template match="kernelextensions">
  
    <xsl:call-template name="header"/>
    
    <xsl:for-each select="directory">
      <p>
        <xsl:call-template name="style"/>
        <xsl:value-of select="path_safe"/>
      </p>
      <xsl:for-each select="extensions/extension">
        <xsl:variable name="status" select="status"/>
    
        <p>
          <xsl:call-template name="style"/>
          <xsl:text>[</xsl:text>
          <xsl:value-of 
            select="$strings/kernelextensions/status/value[@key = $status]"/>
          <xsl:text>]</xsl:text>
          <xsl:text> </xsl:text>
          <xsl:value-of select="filename"/>
          <xsl:text> (</xsl:text>
          <xsl:value-of select="version"/>
          
          <xsl:if test="osversion">
            <xsl:text> - </xsl:text>
            <xsl:value-of select="osversion"/>
          </xsl:if>
          
          <xsl:text>)</xsl:text>
        </p>
      </xsl:for-each>
    </xsl:for-each>
        
  </xsl:template>

  <!-- Print startup items. -->
  <xsl:template match="startupitems">
  
    <xsl:if test="count(startupitem) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:for-each select="startupitem">
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="name"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="$strings/path"/>
          <xsl:value-of select="path"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="version"/>
        </p>
      </xsl:for-each>
      
    </xsl:if>
        
  </xsl:template>

  <!-- Print system launch agents. -->
  <xsl:template match="systemlaunchagents">
  
    <xsl:call-template name="header"/>
    
    <table>
      <xsl:call-template name="tablestyle"/>

      <xsl:apply-templates select="applelaunchdfile"/>
    </table>
    
  </xsl:template>
  
  <!-- Print system launch daemons. -->
  <xsl:template match="systemlaunchdaemons">
  
    <xsl:call-template name="header"/>
    
    <table>
      <xsl:call-template name="tablestyle"/>

      <xsl:apply-templates select="applelaunchdfile"/>
    </table>
        
  </xsl:template>

  <!-- Print Apple tasks. -->
  <xsl:template match="applelaunchdfile">
  
    <xsl:variable name="status" select="status"/>
    
    <tr>
      <td>
        <xsl:call-template name="cellstyle">
          <xsl:with-param name="css">white-space: nowrap;padding-right: 10px;</xsl:with-param>
        </xsl:call-template>
        
        <xsl:text>[</xsl:text>
        <xsl:value-of select="$strings/launchd/status/value[@key = $status]"/>
        <xsl:text>] </xsl:text>
      </td>
      
      <td>
        <xsl:call-template name="cellstyle"/>

        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings"/>
          <xsl:with-param name="key">
            <xsl:text>apple_tasks</xsl:text>
          </xsl:with-param>
          <xsl:with-param name="value" select="count"/>
        </xsl:call-template>
      </td>
    </tr>
        
  </xsl:template>

  <!-- Print launch agents. -->
  <xsl:template match="launchagents">
  
    <xsl:if test="count(launchdfile) &gt; 0">
      <xsl:call-template name="header"/>
      
      <table>
        <xsl:call-template name="tablestyle"/>

        <xsl:apply-templates/>
      </table>
      
    </xsl:if>
        
  </xsl:template>

  <!-- Print launch daemons. -->
  <xsl:template match="launchdaemons">
  
    <xsl:if test="count(launchdfile) &gt; 0">
      <xsl:call-template name="header"/>
      
      <table>
        <xsl:call-template name="tablestyle"/>

        <xsl:apply-templates/>
      </table>

    </xsl:if>
        
  </xsl:template>

  <!-- Print user launch agents. -->
  <xsl:template match="userlaunchagents">
  
    <xsl:if test="count(launchdfile) &gt; 0">
      <xsl:call-template name="header"/>
      
      <table>
        <xsl:call-template name="tablestyle"/>

        <xsl:apply-templates/>
      </table>

    </xsl:if>
        
  </xsl:template>

  <!-- Print a launchd task. -->
  <xsl:template match="launchdfile">
  
    <xsl:variable name="status" select="status"/>

    <tr>
      <td>
        <xsl:call-template name="cellstyle">
          <xsl:with-param name="css">white-space: nowrap;padding-right: 10px;</xsl:with-param>
        </xsl:call-template>
      
        <xsl:text>[</xsl:text>
        <xsl:value-of select="$strings/launchd/status/value[@key = $status]"/>
        <xsl:text>] </xsl:text>
      </td>
      
      <td>
        <xsl:call-template name="cellstyle"/>

        <xsl:value-of select="filename"/>
        <xsl:text> (</xsl:text>
        <xsl:value-of select="author"/>
        
        <xsl:choose>
        
          <xsl:when test="author = 'Shell Script'">
            <xsl:text> </xsl:text>
            <xsl:value-of select="execrc"/>
          </xsl:when>
          
          <xsl:when test="author = '?'">
            <xsl:text> </xsl:text>
            <xsl:value-of select="plistcrc"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="execrc"/>
          </xsl:when>
          
          <xsl:otherwise>
          </xsl:otherwise>
        
        </xsl:choose>
        
        <xsl:text> - </xsl:text>
        <xsl:value-of select="$strings/installed"/>
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="installdate"/>
        <xsl:text>)</xsl:text>
      </td>
    </tr>
    
  </xsl:template>

  <!-- Print login items. -->
  <xsl:template match="loginitems">
  
    <xsl:if test="count(loginitem) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:apply-templates select="loginitem"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Print a single login item. -->
  <xsl:template match="loginitem">
  
    <p>
      <xsl:call-template name="style"/>
      <xsl:value-of select="name"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="type"/>
      
      <xsl:if test="signature or installdate">
      
        <xsl:text> (</xsl:text>
        
        <xsl:choose>
        
          <xsl:when test="signature = 'none'">
          </xsl:when>
          
          <xsl:when test="signature">
            <xsl:value-of select="signature"/>
          </xsl:when>

          <xsl:otherwise>
          </xsl:otherwise>
        
        </xsl:choose>
        
        <xsl:if test="installdate">

          <xsl:choose>
          
            <xsl:when test="signature = 'none'">
            </xsl:when>
            
            <xsl:when test="signature">
              <xsl:text> - </xsl:text>
            </xsl:when>

            <xsl:otherwise>
            </xsl:otherwise>
          
          </xsl:choose>

          <xsl:value-of select="$strings/installed"/>
          <xsl:text> </xsl:text>
          <xsl:apply-templates select="installdate"/>
          <xsl:text>)</xsl:text>
        </xsl:if>
        
      </xsl:if>
    </p>
    
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent">20</xsl:with-param>
      </xsl:call-template>
      <xsl:text> (</xsl:text>
      <xsl:value-of select="path_safe"/>
      <xsl:text>)</xsl:text>
    </p>
        
  </xsl:template>

  <!-- Print internet plugins. -->
  <xsl:template match="internetplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Print user internet plugins. -->
  <xsl:template match="userinternetplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Print audio plugins. -->
  <xsl:template match="audioplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Print user audio plugins. -->
  <xsl:template match="useraudioplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Print iTunes plugins. -->
  <xsl:template match="itunesplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Print user iTunes plugins. -->
  <xsl:template match="useritunesplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Print a single plugin. -->
  <xsl:template match="plugin">
  
    <p>
      <xsl:call-template name="style"/>
      <xsl:value-of select="name"/>
      <xsl:text>: </xsl:text>
      <xsl:value-of select="version"/>
      
      <xsl:if test="installdate">
        <xsl:text> (</xsl:text>
        <xsl:value-of select="$strings/installed"/>
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="installdate"/>
        <xsl:text>)</xsl:text>
      </xsl:if>
    </p>
        
  </xsl:template>

  <!-- Print Safari extensions. -->
  <xsl:template match="safariextensions">
  
    <xsl:if test="count(extension) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>

        <xsl:apply-templates select="extension"/>
      </table>
      
    </xsl:if>
        
  </xsl:template>
  
  <!-- Print a single Safari extension. -->
  <xsl:template match="extension">
  
    <xsl:variable name="status" select="status"/>

    <tr>
      <td>
        <xsl:call-template name="cellstyle"/>

        <xsl:value-of select="name"/>
        <xsl:text> - </xsl:text>
        <xsl:value-of select="developer"/>
        <xsl:text> - </xsl:text>
        <a target="_blank">
          <xsl:attribute name="href">
            <xsl:value-of select="url"/>
          </xsl:attribute>
          <strong>
            <xsl:value-of select="url"/>
          </strong>
        </a>
        <xsl:text> (</xsl:text>
        <xsl:value-of select="$strings/installed"/>
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="installdate"/>
        <xsl:text>)</xsl:text>
      </td>
    </tr>
        
  </xsl:template>

  <!-- Print Preference panes. -->
  <xsl:template match="preferencepanes">
  
    <xsl:if test="count(preferencepane) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="preferencepane"/>
    </xsl:if>
        
  </xsl:template>
  
  <!-- Print a single Preference pane. -->
  <xsl:template match="preferencepane">
  
    <p>
      <xsl:call-template name="style"/>
      <xsl:value-of select="name"/>
      <xsl:text> (</xsl:text>
      <xsl:value-of select="$strings/installed"/>
      <xsl:text> </xsl:text>
      <xsl:apply-templates select="installdate"/>
      <xsl:text>)</xsl:text>
    </p>
        
  </xsl:template>

  <!-- Print Fonts. -->
  <xsl:template match="fonts">
  
    <!-- This should only be bad fonts. -->
    <xsl:if test="count(fonts) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="font"/>
    </xsl:if>
        
  </xsl:template>
  
  <!-- Print a single font. -->
  <xsl:template match="font">
  
    <p>
      <xsl:call-template name="style"/>
      <xsl:text>[</xsl:text>
      <xsl:value-of select="@status"/>
      <xsl:text>] </xsl:text>
      <xsl:value-of select="name"/>
      <xsl:text> - </xsl:text>
      <xsl:value-of select="date"/>
      <xsl:text>)</xsl:text>
    </p>
        
  </xsl:template>

  <!-- Print Time Machine information. -->
  <xsl:template match="timemachine">
  
    <xsl:call-template name="header"/>

    <xsl:choose>
      <xsl:when test="notconfigured = 'true'">
      
        <p>
          <xsl:call-template name="style"/>
          
          <strong>
            <xsl:value-of select="$strings/tm/notconfigured"/>
          </strong>
        </p>
        
      </xsl:when>
      
      <xsl:otherwise>
      
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="$strings/tm/skip_system_files"/>
          <xsl:apply-templates select="skipsystemfiles"/>
        </p>
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="$strings/tm/mobile_backups"/>
          <xsl:apply-templates select="mobilebackups"/>
        </p>
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="$strings/tm/auto_backup"/>
          <xsl:apply-templates select="autobackup"/>
        </p>
        
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="$strings/tm/volumes_backed_up"/>
        </p>

        <xsl:apply-templates 
          select="volumesbeingbackedup/volume" mode="timemachine"/>
        
        <p>
          <xsl:call-template name="style"/>
          <xsl:value-of select="$strings/tm/destinations"/>
        </p>

        <xsl:apply-templates select="destinations/destination"/>
        
      </xsl:otherwise>
   
    </xsl:choose>
    
  </xsl:template>

  <!-- Print a Time Machine volume. -->
  <xsl:template match="volume" mode="timemachine">
  
    <p>
      <xsl:call-template name="style"/>
      <xsl:value-of select="cleanname"/>
      <xsl:text>: </xsl:text>
      <xsl:value-of select="$strings/tm/disk_size"/>
      <xsl:apply-templates select="size" mode="units"/>
      <xsl:text> - </xsl:text>
      <xsl:value-of select="$strings/tm/disk_used"/>
      <xsl:apply-templates select="used" mode="units"/>
      <xsl:text> </xsl:text>
    </p>
      
  </xsl:template>

  <!-- Print a Time Machine destination. -->
  <xsl:template match="destination">
  
    <p>
      <xsl:call-template name="style"/>
      <xsl:value-of select="cleanname"/>
      <xsl:text> [</xsl:text>
      <xsl:value-of select="type"/>
      <xsl:text>]</xsl:text>
      
      <xsl:if test="lastused = 'true'">
        <xsl:text> </xsl:text>
        <xsl:value-of select="$strings/tm/last_used"/>
      </xsl:if>
    </p>
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent">20</xsl:with-param>
      </xsl:call-template>
      <xsl:value-of select="$strings/tm/total_size"/>
      <xsl:apply-templates select="size" mode="units"/>
    </p>
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent">20</xsl:with-param>
      </xsl:call-template>
      <xsl:value-of select="$strings/tm/number_of_backups"/>
      <xsl:value-of select="count"/>
    </p>
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent">20</xsl:with-param>
      </xsl:call-template>
      <xsl:value-of select="$strings/tm/oldest_backup"/>
      <xsl:value-of select="oldestbackup"/>
    </p>
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent">20</xsl:with-param>
      </xsl:call-template>
      <xsl:value-of select="$strings/tm/last_backup"/>
      <xsl:value-of select="lastbackup"/>
    </p>
    
  </xsl:template>

  <!-- Print a CPU usage information. -->
  <xsl:template match="cpu">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>
        
        <tr>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong><xsl:value-of select="$strings/cpu/process"/></strong>
          </td>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong>
              <xsl:value-of select="$strings/cpu/source"/>
            </strong>
          </td>
          <td>
            <xsl:call-template name="unitsstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong><xsl:value-of select="$strings/cpu/usage"/></strong>
          </td>
        </tr>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <tr>
            <td>
              <xsl:call-template name="cellstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>
              
              <xsl:value-of select="name"/>
              <xsl:if test="count &gt; 1">
                <xsl:value-of select="concat(' (', count, ')')"/>
              </xsl:if>
            </td>
            <td>
              <xsl:call-template name="cellstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="$source"/>
            </td>
            <td>
              <xsl:call-template name="unitsstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>
              
              <xsl:apply-templates select="usage" mode="units"/>
            </td>
          </tr>
        </xsl:for-each>
      </table>
    </xsl:if>
      
  </xsl:template>

  <!-- Print a memory usage information. -->
  <xsl:template match="memory">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>
        
        <tr>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong><xsl:value-of select="$strings/memory/process"/></strong>
          </td>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong>
              <xsl:value-of select="$strings/memory/source"/>
            </strong>
          </td>
          <td>
            <xsl:call-template name="unitsstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong><xsl:value-of select="$strings/memory/usage"/></strong>
          </td>
        </tr>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <tr>
            <td>
              <xsl:call-template name="cellstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="name"/>
              <xsl:if test="count &gt; 1">
                <xsl:value-of select="concat(' (', count, ')')"/>
              </xsl:if>
            </td>
            <td>
              <xsl:call-template name="cellstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>
              
              <xsl:value-of select="$source"/>
            </td>
            <td>
              <xsl:call-template name="unitsstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:apply-templates select="size" mode="units"/>
            </td>
          </tr>
        </xsl:for-each>
      </table>
    </xsl:if>
      
  </xsl:template>

  <!-- Print a network usage information. -->
  <xsl:template match="networkusage">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>
        
        <tr>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong><xsl:value-of select="$strings/networkusage/process"/></strong>
          </td>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong>
              <xsl:value-of select="$strings/networkusage/source"/>
            </strong>
          </td>
          <td>
            <xsl:call-template name="unitsstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong><xsl:value-of select="$strings/networkusage/input"/></strong>
          </td>
          <td>
            <xsl:call-template name="unitsstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong><xsl:value-of select="$strings/networkusage/output"/></strong>
          </td>
        </tr>
        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <tr>
            <td>
              <xsl:call-template name="cellstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="name"/>
            </td>
            <td>
              <xsl:call-template name="cellstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="$source"/>
            </td>
            <td>
              <xsl:call-template name="unitsstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:apply-templates select="inputsize" mode="units"/>
            </td>
            <td>
              <xsl:call-template name="unitsstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>
              
              <xsl:apply-templates select="outputsize" mode="units"/>
            </td>
          </tr>
        </xsl:for-each>
      </table>
    </xsl:if>
      
  </xsl:template>

  <!-- Print a energy usage information. -->
  <xsl:template match="energy">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>
        
        <tr>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong><xsl:value-of select="$strings/energy/process"/></strong>
          </td>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong>
              <xsl:value-of select="$strings/energy/source"/>
            </strong>
          </td>
          <td>
            <xsl:call-template name="unitsstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>

            <strong><xsl:value-of select="$strings/energy/usage"/></strong>
          </td>
        </tr>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <tr>
            <td>
              <xsl:call-template name="cellstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>
              
              <xsl:value-of select="name"/>
              <xsl:if test="count &gt; 1">
                <xsl:value-of select="concat(' (', count, ')')"/>
              </xsl:if>
            </td>
            <td>
              <xsl:call-template name="cellstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="$source"/>
            </td>
            <td>
              <xsl:call-template name="unitsstyle">
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>
              
              <xsl:value-of select="energy"/>
            </td>
          </tr>
        </xsl:for-each>
      </table>
    </xsl:if>
      
  </xsl:template>

  <!-- Print a virtual memory usage information. -->
  <xsl:template match="vm">
  
    <xsl:call-template name="header"/>

    <table>
      <xsl:call-template name="tablestyle"/>
      
      <tr>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>

          <xsl:value-of select="$strings/available_ram"/>
        </td>
        <td>
          <xsl:call-template name="unitsstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>

          <xsl:apply-templates select="availableram" mode="units"/>
        </td>
      </tr>
      <tr>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>

          <xsl:value-of select="$strings/free_ram"/>
        </td>
        <td>
          <xsl:call-template name="unitsstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>

          <xsl:apply-templates select="freeram" mode="units"/>
        </td>
      </tr>
      <tr>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>

          <xsl:value-of select="$strings/used_ram"/>
        </td>
        <td>
          <xsl:call-template name="unitsstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>

          <xsl:apply-templates select="usedram" mode="units"/>
        </td>
      </tr>
      <tr>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>

          <xsl:value-of select="$strings/cached_files"/>
        </td>
        <td>
          <xsl:call-template name="unitsstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>

          <xsl:apply-templates select="filecache" mode="units"/>
        </td>
      </tr>
      <tr>
        <td>
          <xsl:call-template name="cellstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>

          <xsl:value-of select="$strings/swap_used"/>
        </td>
        <td>
          <xsl:call-template name="unitsstyle">
            <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
          </xsl:call-template>

          <xsl:apply-templates select="swapused" mode="units"/>
        </td>
      </tr>
    </table>
      
  </xsl:template>

  <!-- Print software install information. -->
  <xsl:template match="install">
  
    <xsl:if test="count(package) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>
      
        <tr>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>
            
            <strong><xsl:value-of select="$strings/install/name"/></strong>
          </td>
          <td>
            <xsl:call-template name="cellstyle">
              <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
            </xsl:call-template>
            
            <strong><xsl:value-of select="$strings/install/version"/></strong>
          </td>
          <td>
            <xsl:call-template name="cellstyle"/>
            
            <strong><xsl:value-of select="$strings/install/date"/></strong>
          </td>
        </tr>
        
        <xsl:apply-templates select="package"/>
        
      </table>
    </xsl:if>
   
  </xsl:template>

  <!-- Print an install package. -->
  <xsl:template match="package">
  
    <tr>
      <td>
        <xsl:call-template name="cellstyle">
          <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
        </xsl:call-template>
        
        <xsl:value-of select="name"/>
      </td>
      
      <td>
        <xsl:call-template name="cellstyle">
          <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
        </xsl:call-template>
        
        <xsl:if test="version">
          <xsl:value-of select="version"/>
        </xsl:if>
      </td>
      
      <td>
        <xsl:call-template name="cellstyle"/>
        
        <xsl:apply-templates select="installdate"/>
      </td>
    </tr>
    
  </xsl:template>

  <!-- Print diagnostics information. -->
  <xsl:template match="diagnostics">
  
    <xsl:call-template name="header"/>
    
    <xsl:apply-templates select="event"/>
    
    <xsl:if test="insufficientpermissions = 'true'">
      
      <xsl:call-template name="insufficientpermissions"/>
      
    </xsl:if>
        
  </xsl:template>

  <!-- Print insufficient permissions. -->
  <xsl:template name="insufficientpermissions">
  
    <br/>
    
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent">10</xsl:with-param>
      </xsl:call-template>
      
      <strong>
        <xsl:copy-of 
          select="$strings/diagnostics/insufficientpermissions/node()"/>
      </strong>
    </p>
    <p>
      <xsl:call-template name="style">
        <xsl:with-param name="indent">10</xsl:with-param>
      </xsl:call-template>
      
      <xsl:apply-templates 
        select="$strings/diagnostics/runasadmin" mode="label"/>
    </p>

  </xsl:template>
  
  <!-- Print a diagnostics event. -->
  <xsl:template match="event">
  
    <xsl:variable name="type" select="type"/>
    
    <p>
      <xsl:call-template name="style"/>
      
      <xsl:value-of select="date"/>
      <xsl:text> </xsl:text>
      
      <xsl:choose>
        
        <xsl:when test="type = 10">
          <xsl:value-of select="$strings/diagnostics/types/value[@key = $type]"/>
          <xsl:text> </xsl:text>
          <xsl:apply-templates select="name"/>
        </xsl:when>
        
        <xsl:otherwise>
          <xsl:apply-templates select="name"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="$strings/diagnostics/types/value[@key = $type]"/>
        </xsl:otherwise>
      
      </xsl:choose>
      
    </p>
    
    <xsl:if test="information">
      <table>
        <xsl:call-template name="tablestyle"/>
        
        <tr>
          <td>
            <!-- ASC scrambles multi-line elements. Do the scrambling myself 
                 so I have some control over the outcome. -->
            <xsl:call-template name="split">
              <xsl:with-param name="text">
                <xsl:value-of select="information"/>
              </xsl:with-param>
            </xsl:call-template>
          </td>
        </tr>
      </table>
    </xsl:if>
    
  </xsl:template>
  
  <!-- Print a EtreCheck deleted files. -->
  <xsl:template match="etrecheckdeletedfiles">
  
    <xsl:if test="count(deletedfile) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>
        
        <xsl:for-each select="deletedfile">
          <tr>
            <td>
              <xsl:call-template name="cellstyle">          
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="date"/>
            </td>
            <td>
              <xsl:call-template name="cellstyle">          
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="path_safe"/>
            </td>
            <td>
              <xsl:call-template name="cellstyle">          
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="reason"/>
            </td>
          </tr>
        </xsl:for-each>
        
      </table>
      
    </xsl:if>
      
  </xsl:template>

  <!-- Print a device node. -->
  <xsl:template match="node">

    <p>
      <xsl:call-template name="style"/>
      
      <xsl:if test="manufacturer">
        <xsl:value-of select="manufacturer"/>
        <xsl:text> </xsl:text>
      </xsl:if>
      
      <xsl:if test="name">
        <xsl:value-of select="name"/>
      </xsl:if>
    </p>

    <xsl:apply-templates select="node"/>
    
  </xsl:template>
  
  <!-- Create an HTML CSS style attribute suitable for a text report. -->
  <xsl:template name="style">
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="color"/>
    <xsl:param name="style"/>
    <xsl:param name="css"/>
  
    <xsl:attribute name="style">
    
      <xsl:text>text-indent: </xsl:text>
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">
          <xsl:value-of select="$indent"/>
        </xsl:with-param>
      </xsl:call-template>
      <xsl:text>px;</xsl:text>
      
      <xsl:call-template name="base_style">
        <xsl:with-param name="color" select="$color"/>
        <xsl:with-param name="color" select="$style"/>
        <xsl:with-param name="color" select="$css"/>
      </xsl:call-template>

    </xsl:attribute>

  </xsl:template>
  
  <!-- Create an HTML CSS style attribute suitable for a table cell in the 
       text report. -->
  <xsl:template name="cellstyle">
    <xsl:param name="color"/>
    <xsl:param name="style"/>
    <xsl:param name="css"/>
  
    <xsl:attribute name="style">
    
      <xsl:call-template name="base_style">
        <xsl:with-param name="color">
          <xsl:value-of select="$color"/>
        </xsl:with-param>
        <xsl:with-param name="style">
          <xsl:value-of select="$style"/>
        </xsl:with-param>
        <xsl:with-param name="css">
          <xsl:value-of select="$css"/>
        </xsl:with-param>
      </xsl:call-template>

    </xsl:attribute>

  </xsl:template>

  <!-- Print a number with units right aligned. -->
  <xsl:template name="unitsstyle">
    <xsl:param name="css"/>
  
    <xsl:attribute name="style">

      <xsl:call-template name="base_style">
        <xsl:with-param name="css">
          <xsl:value-of select="concat($css,'text-align: right; ')"/>
        </xsl:with-param>
      </xsl:call-template>

    </xsl:attribute>

  </xsl:template>

  <!-- Create an HTML CSS style attribute suitable for a text report. -->
  <xsl:template name="base_style">
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="color"/>
    <xsl:param name="style"/>
    <xsl:param name="css"/>
  
    <xsl:if test="string-length($color) &gt; 0">
      <xsl:text>color: </xsl:text>
      <xsl:value-of select="$color"/>
      <xsl:text>;</xsl:text>
    </xsl:if>
    
    <xsl:if test="$style = 'bold'">
      <xsl:text>font-weight: bold;</xsl:text>
    </xsl:if>

    <xsl:if test="$style = 'italic'">
      <xsl:text>font-style: italic;</xsl:text>
    </xsl:if>

    <xsl:text>font-size: 12px; font-family: Helvetica, Arial, sans-serif; margin: 0;</xsl:text>
    <xsl:value-of select="$css"/>
      
  </xsl:template>

  <!-- Calculate the indent. -->
  <xsl:template name="indent">
    <xsl:param name="indent">0</xsl:param>
  
    <xsl:variable name="depth" select="count(ancestor::node())"/>
    
    <xsl:choose>
    
      <xsl:when test="ancestor-or-self::header">
        <xsl:value-of select="$indent"/>
      </xsl:when>
      
      <xsl:when test="ancestor-or-self::hardware">
        <xsl:value-of select="$indent + (10 * $depth)"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::systemsoftware">
        <xsl:value-of select="$indent + (10 * $depth)"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::configurationfiles">
        <xsl:value-of select="$indent + (10 * $depth)"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::security">
        <xsl:value-of select="$indent + (10 * $depth)"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::timemachine">
        <xsl:value-of select="$indent + (10 * $depth)"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::cleanup">
        <xsl:value-of select="$indent + (10 * $depth) - 20"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::cpu">
        <xsl:value-of select="$indent"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::memory">
        <xsl:value-of select="$indent"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::networkusage">
        <xsl:value-of select="$indent"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::energy">
        <xsl:value-of select="$indent"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::vm">
        <xsl:value-of select="$indent"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::etrecheckdeletedfiles">
        <xsl:value-of select="$indent"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::launchdfile">
        <xsl:value-of select="$indent"/>
      </xsl:when>

      <xsl:when test="ancestor-or-self::safari">
        <xsl:value-of select="$indent"/>
      </xsl:when>

      <xsl:otherwise>
        <xsl:value-of select="$indent + (10 * $depth) - 10"/>
      </xsl:otherwise>
    
    </xsl:choose>
    
  </xsl:template>
  
  <!-- Apply a table style to match other output. -->
  <xsl:template name="tablestyle">
    <xsl:param name="margin-left">20px</xsl:param>
    
    <xsl:attribute name="style">
      <xsl:text>color: #000000; border-spacing: 0; margin-left: </xsl:text>
      <xsl:value-of select="$margin-left"/>
      <xsl:text>;</xsl:text>
    </xsl:attribute>

  </xsl:template>

  <!-- ASC scrambles embedded new lines. Can't fix it with CSS. -->
  <xsl:template name="split">
    <xsl:param name="text"/>

    <xsl:if test="string-length($text) &gt; 0">
      
      <xsl:choose>
      
        <xsl:when test="contains($text, $newline)">
          <xsl:variable name="before">
            <xsl:value-of select="substring-before($text, $newline)"/>
          </xsl:variable>

          <xsl:variable name="after">
            <xsl:value-of select="substring-after($text, $newline)"/>
          </xsl:variable>

          <xsl:if test="normalize-space($before) != ''">
            <p>
              <xsl:call-template name="style"/>
              <xsl:value-of select="$before"/>
            </p>
          </xsl:if>

          <xsl:call-template name="split">
            <xsl:with-param name="text" select="$after"/>
          </xsl:call-template>
        </xsl:when>
        
        <xsl:otherwise>
          <p>
            <xsl:call-template name="style"/>
            <xsl:value-of select="$text"/>
          </p>
        </xsl:otherwise>
      
      </xsl:choose>
      
    </xsl:if>
  </xsl:template>

  <!-- Print a section header. -->
  <xsl:template name="header">
    <xsl:param name="key"><xsl:value-of select="local-name(.)"/></xsl:param>
    
    <xsl:call-template name="custom_header">
      <xsl:with-param 
        name="text" select="$strings/headers/value[@key = $key]"/>
    </xsl:call-template>
    
  </xsl:template>
  
  <!-- Print a custom section header. -->
  <xsl:template name="custom_header">
    <xsl:param name="text"/>
    
    <br/>
    <p style="font-size: 12px; font-family: Helvetica, Arial, sans-serif; margin: 0;">
      <strong>
        <xsl:value-of select="$text"/>
      </strong>
    </p>
    
  </xsl:template>

  <!-- Print only the day portion of an install date. -->
  <xsl:template match="installdate">
  
    <xsl:value-of select="substring(., 1, 10)"/>
    
  </xsl:template>
      
  <!-- Print a label. -->
  <xsl:template match="*" mode="label">
  
    <strong>
      <xsl:copy-of select="*|text()"/>
    </strong>
    
  </xsl:template>
    
  <xsl:include href="Library.xsl" />

</xsl:stylesheet>
